@@ -1,9 +1,15 @@
+use std::ops::Deref;
+
 use proc_macro2::{Ident, Span, TokenStream};
-use quote::quote_spanned;
+use quote::{quote, quote_spanned};
+use syn::parse::Parser;
+use syn::punctuated::Punctuated;
+use syn::{Expr, ExprBlock};
 use syn::{parse::Error, spanned::Spanned};
 
 use crate::mutator::Mutator;
-use crate::util::{expr_to_lit_string, ident_to_type, path_to_single_string, strip_raw_ident_prefix, ApplyMeta, AttrArg};
+use crate::struct_info::StructInfo;
+use crate::util::{ApplyMeta, AttrArg, expr_to_lit_string, ident_to_type, path_to_single_string, strip_raw_ident_prefix};
 
 #[derive(Debug)]
 pub struct FieldInfo<'a> {
@@ -39,7 +45,7 @@ impl<'a> FieldInfo<'a> {
     }
 
     pub fn tuplized_type_ty_param(&self) -> syn::Type {
-        let mut types = syn::punctuated::Punctuated::default();
+        let mut types = Punctuated::default();
         types.push(self.ty.clone());
         types.push_punct(Default::default());
         syn::TypeTuple {
@@ -50,7 +56,13 @@ impl<'a> FieldInfo<'a> {
     }
 
     pub fn type_from_inside_option(&self) -> Option<&syn::Type> {
-        let path = if let syn::Type::Path(type_path) = self.ty {
+        let typ = if let syn::Type::Group(type_group) = self.ty {
+            type_group.elem.deref()
+        } else {
+            self.ty
+        };
+
+        let path = if let syn::Type::Path(type_path) = typ {
             if type_path.qself.is_some() {
                 return None;
             }
@@ -106,13 +118,93 @@ impl<'a> FieldInfo<'a> {
                 }),
             }));
         }
+        if self.builder_attr.default.is_none()
+            && let Some(default_where) = self.builder_attr.default_where.as_ref()
+        {
+            return Err(Error::new(
+                default_where.span(),
+                "default_where is not allowed without a default",
+            ));
+        }
         Ok(self)
     }
+
+    pub fn maybe_mut(&self) -> TokenStream {
+        if let Some(span) = self.builder_attr.mutable_during_default_resolution {
+            quote_spanned!(span => mut)
+        } else {
+            quote!()
+        }
+    }
+
+    pub fn gen_next_field_default_trait_impl(&self, struct_info: &StructInfo) -> syn::Result<Option<TokenStream>> {
+        if self.builder_attr.setter.skip.is_some() {
+            return Ok(None);
+        }
+        let Some(default_expr) = self.builder_attr.default.as_ref() else {
+            return Ok(None);
+        };
+
+        let crate_module_path = &struct_info.builder_attr.crate_module_path;
+        let struct_name = struct_info.name;
+        let (impl_generics, ty_generics, where_clause) = struct_info.generics.split_for_impl();
+        let field_type = self.ty;
+
+        let (dep_types, dep_names): (Vec<_>, Vec<_>) = struct_info
+            .fields
+            .iter()
+            .take(self.ordinal)
+            .map(|dep_field| {
+                let dep_type = dep_field.ty;
+                let dep_mut = dep_field.maybe_mut();
+                (quote!(&#dep_mut #dep_type), dep_field.name)
+            })
+            .unzip();
+
+        let where_clause_storage;
+        let where_clause_for_default = if let Some(default_where) = self.builder_attr.default_where.as_ref() {
+            let mut predicates: Punctuated<_, _> = Default::default();
+            if let Some(where_clause) = where_clause {
+                predicates.extend(where_clause.predicates.iter().cloned());
+            }
+            predicates.extend(default_where.iter().cloned());
+            where_clause_storage = syn::WhereClause {
+                where_token: Default::default(),
+                predicates,
+            };
+            Some(&where_clause_storage)
+        } else {
+            where_clause
+        };
+
+        Ok(Some(quote! {
+            #[allow(clippy::ref_option_ref)]
+            #[automatically_derived]
+            impl #impl_generics #crate_module_path::NextFieldDefault<(#(#dep_types,)* (#field_type,),)> for #struct_name #ty_generics #where_clause {
+                type Output = #field_type;
+
+                fn resolve((.., (input,),): (#(#dep_types,)* (#field_type,),)) -> Self::Output {
+                    input
+                }
+            }
+
+            #[allow(clippy::ref_option_ref)]
+            #[automatically_derived]
+            impl #impl_generics #crate_module_path::NextFieldDefault<(#(#dep_types,)* (),)> for #struct_name #ty_generics #where_clause_for_default {
+                type Output = #field_type;
+
+                fn resolve((#(#dep_names,)* (),): (#(#dep_types,)* (),)) -> Self::Output {
+                    #default_expr
+                }
+            }
+        }))
+    }
 }
 
 #[derive(Debug, Default, Clone)]
 pub struct FieldBuilderAttr<'a> {
     pub default: Option<syn::Expr>,
+    pub default_where: Option<Punctuated<syn::WherePredicate, syn::token::Comma>>,
     pub via_mutators: Option<ViaMutators>,
     pub deprecated: Option<&'a syn::Attribute>,
     pub doc_comments: Vec<&'a syn::Expr>,
@@ -240,7 +332,18 @@ impl ApplyMeta for FieldBuilderAttr<'_> {
                     Ok(())
                 }
                 AttrArg::Sub(_) => Err(expr.incorrect_type()),
+                AttrArg::Fn(_) => Err(expr.incorrect_type()),
             },
+            "default_where" => {
+                let sub_attr = expr.sub_attr()?;
+                let span = sub_attr.args.span();
+                self.default_where = Some(
+                    Punctuated::parse_terminated
+                        .parse2(sub_attr.args)
+                        .map_err(|e| Error::new(span, format!("{e}")))?,
+                );
+                Ok(())
+            }
             "default_code" => {
                 use std::str::FromStr;
 
@@ -287,6 +390,7 @@ impl ApplyMeta for FieldBuilderAttr<'_> {
                             self.via_mutators = Some(via_mutators);
                         }
                     }
+                    AttrArg::Fn(_) => return Err(expr.incorrect_type()),
                 }
                 Ok(())
             }
@@ -310,10 +414,13 @@ impl ApplyMeta for SetterSettings {
                 Ok(())
             }
             "transform" => {
-                self.transform = if let Some(key_value) = expr.key_value_or_not()? {
-                    Some(parse_transform_closure(key_value.name.span(), key_value.parse_value()?)?)
-                } else {
-                    None
+                self.transform = match expr {
+                    AttrArg::Fn(func) => Some(parse_transform_fn(func.span(), func)?),
+                    AttrArg::KeyValue(key_value) => {
+                        Some(parse_transform_closure(key_value.name.span(), key_value.parse_value()?)?)
+                    }
+                    AttrArg::Not { .. } => None,
+                    _ => return Err(expr.incorrect_type()),
                 };
                 Ok(())
             }
@@ -354,12 +461,21 @@ impl ApplyMeta for SetterSettings {
 #[derive(Debug, Clone)]
 pub struct Strip {
     pub fallback: Option<syn::Ident>,
+    pub fallback_prefix: Option<String>,
+    pub fallback_suffix: Option<String>,
+    pub ignore_invalid: bool,
     span: Span,
 }
 
 impl Strip {
     fn new(span: Span) -> Self {
-        Self { fallback: None, span }
+        Self {
+            fallback: None,
+            fallback_prefix: None,
+            fallback_suffix: None,
+            ignore_invalid: false,
+            span,
+        }
     }
 }
 
@@ -378,9 +494,43 @@ impl ApplyMeta for Strip {
                 self.fallback = Some(ident);
                 Ok(())
             }
+            "fallback_prefix" => {
+                if self.fallback_prefix.is_some() {
+                    return Err(Error::new_spanned(
+                        expr.name(),
+                        format!("Duplicate fallback_prefix parameter {:?}", expr.name().to_string()),
+                    ));
+                }
+
+                self.fallback_prefix = Some(expr.key_value()?.parse_value::<syn::LitStr>()?.value());
+                Ok(())
+            }
+            "fallback_suffix" => {
+                if self.fallback_suffix.is_some() {
+                    return Err(Error::new_spanned(
+                        expr.name(),
+                        format!("Duplicate fallback_suffix parameter {:?}", expr.name().to_string()),
+                    ));
+                }
+
+                self.fallback_suffix = Some(expr.key_value()?.parse_value::<syn::LitStr>()?.value());
+                Ok(())
+            }
+            "ignore_invalid" => {
+                if self.ignore_invalid {
+                    return Err(Error::new_spanned(
+                        expr.name(),
+                        format!("Duplicate ignore_invalid parameter {:?}", expr.name().to_string()),
+                    ));
+                }
+
+                expr.flag()?;
+                self.ignore_invalid = true;
+                Ok(())
+            }
             _ => Err(Error::new_spanned(
                 expr.name(),
-                format!("Invalid parameter used {:?}", expr.name().to_string()),
+                format!("Unknown parameter {:?}", expr.name().to_string()),
             )),
         }
     }
@@ -390,14 +540,47 @@ impl ApplyMeta for Strip {
 pub struct Transform {
     pub params: Vec<(syn::Pat, syn::Type)>,
     pub body: syn::Expr,
+    pub generics: Option<syn::Generics>,
+    pub return_type: syn::ReturnType,
     span: Span,
 }
 
+fn parse_transform_fn(span: Span, func: syn::ItemFn) -> Result<Transform, Error> {
+    if let Some(kw) = &func.sig.asyncness {
+        return Err(Error::new(kw.span, "Transform function cannot be async"));
+    }
+
+    let params = func
+        .sig
+        .inputs
+        .into_iter()
+        .map(|input| match input {
+            syn::FnArg::Typed(pat_type) => Ok((*pat_type.pat, *pat_type.ty)),
+            syn::FnArg::Receiver(_) => Err(Error::new_spanned(input, "Transform function cannot have self parameter")),
+        })
+        .collect::<Result<Vec<_>, _>>()?;
+
+    let body = Expr::Block(ExprBlock {
+        attrs: Vec::new(),
+        label: None,
+        block: *func.block,
+    });
+
+    Ok(Transform {
+        params,
+        body,
+        span,
+        generics: Some(func.sig.generics),
+        return_type: func.sig.output,
+    })
+}
+
 fn parse_transform_closure(span: Span, expr: syn::Expr) -> Result<Transform, Error> {
     let closure = match expr {
         syn::Expr::Closure(closure) => closure,
         _ => return Err(Error::new_spanned(expr, "Expected closure")),
     };
+
     if let Some(kw) = &closure.asyncness {
         return Err(Error::new(kw.span, "Transform closure cannot be async"));
     }
@@ -418,6 +601,8 @@ fn parse_transform_closure(span: Span, expr: syn::Expr) -> Result<Transform, Err
         params,
         body: *closure.body,
         span,
+        generics: None,
+        return_type: closure.output,
     })
 }
 
