@@ -1,23 +1,25 @@
-use proc_macro2::{Ident, Span, TokenStream};
-use quote::{format_ident, quote, quote_spanned, ToTokens};
-use syn::{parse::Error, parse_quote, punctuated::Punctuated, GenericArgument, ItemFn, Token};
+use proc_macro2::TokenStream;
+use quote::{ToTokens, format_ident, quote};
+use syn::{GenericArgument, ItemFn, Token, parse::Error, parse_quote, punctuated::Punctuated};
 
-use crate::builder_attr::{IntoSetting, TypeBuilderAttr};
-use crate::field_info::FieldInfo;
-use crate::mutator::Mutator;
-use crate::util::{
-    empty_type, empty_type_tuple, first_visibility, modify_types_generics_hack, phantom_data_for_generics, public_visibility,
-    strip_raw_ident_prefix, type_tuple,
+use crate::{
+    builder_attr::{IntoSetting, TypeBuilderAttr},
+    field_info::FieldInfo,
+    mutator::Mutator,
+    util::{
+        empty_type, empty_type_tuple, first_visibility, modify_types_generics_hack, phantom_data_for_generics, public_visibility,
+        strip_raw_ident_prefix, type_tuple,
+    },
 };
 
 #[derive(Debug)]
 pub struct StructInfo<'a> {
     vis: &'a syn::Visibility,
-    name: &'a syn::Ident,
-    generics: &'a syn::Generics,
-    fields: Vec<FieldInfo<'a>>,
+    pub name: &'a syn::Ident,
+    pub generics: &'a syn::Generics,
+    pub fields: Vec<FieldInfo<'a>>,
 
-    builder_attr: TypeBuilderAttr<'a>,
+    pub builder_attr: TypeBuilderAttr<'a>,
     builder_name: syn::Ident,
 }
 
@@ -51,6 +53,7 @@ impl<'a> StructInfo<'a> {
         let builder_attr = TypeBuilderAttr::new(&ast.attrs)?;
         let builder_name = builder_attr
             .builder_type
+            .common
             .get_name()
             .map(|name| strip_raw_ident_prefix(name.to_string()))
             .unwrap_or_else(|| strip_raw_ident_prefix(format!("{}Builder", ast.ident)));
@@ -108,7 +111,7 @@ impl<'a> StructInfo<'a> {
         let builder_method_name = self.builder_attr.builder_method.get_name().unwrap_or_else(|| quote!(builder));
         let builder_method_visibility = first_visibility(&[
             self.builder_attr.builder_method.vis.as_ref(),
-            self.builder_attr.builder_type.vis.as_ref(),
+            self.builder_attr.builder_type.common.vis.as_ref(),
             Some(vis),
         ]);
         let builder_method_doc = self.builder_attr.builder_method.get_doc_or(|| {
@@ -140,9 +143,9 @@ impl<'a> StructInfo<'a> {
             )
         });
 
-        let builder_type_visibility = first_visibility(&[self.builder_attr.builder_type.vis.as_ref(), Some(vis)]);
+        let builder_type_visibility = first_visibility(&[self.builder_attr.builder_type.common.vis.as_ref(), Some(vis)]);
         let builder_type_doc = if self.builder_attr.doc {
-            self.builder_attr.builder_type.get_doc_or(|| {
+            self.builder_attr.builder_type.common.get_doc_or(|| {
                 format!(
                     "
                     Builder for [`{name}`] instances.
@@ -165,6 +168,8 @@ impl<'a> StructInfo<'a> {
             b_generics_where.predicates.extend(predicates.predicates.clone());
         }
 
+        let builder_type_attributes = &self.builder_attr.builder_type.attributes;
+
         Ok(quote! {
             #[automatically_derived]
             impl #impl_generics #name #ty_generics #where_clause {
@@ -181,6 +186,7 @@ impl<'a> StructInfo<'a> {
             #[must_use]
             #builder_type_doc
             #[allow(dead_code, non_camel_case_types, non_snake_case)]
+            #(#builder_type_attributes)*
             #builder_type_visibility struct #builder_name #b_generics #b_generics_where_extras_predicates {
                 fields: #all_fields_param,
                 phantom: #phantom_data,
@@ -261,13 +267,28 @@ impl<'a> StructInfo<'a> {
 
         let deprecated = &field.builder_attr.deprecated;
 
-        // NOTE: both auto_into and strip_option affect `arg_type` and `arg_expr`, but the order of
-        // nesting is different, so we have to do this little dance.
+        let option_was_stripped;
         let arg_type = if field.builder_attr.setter.strip_option.is_some() && field.builder_attr.setter.transform.is_none() {
-            field
-                .type_from_inside_option()
-                .ok_or_else(|| Error::new_spanned(field_type, "can't `strip_option` - field is not `Option<...>`"))?
+            if let Some(inner_type) = field.type_from_inside_option() {
+                option_was_stripped = true;
+                inner_type
+            } else if field
+                .builder_attr
+                .setter
+                .strip_option
+                .as_ref()
+                .is_some_and(|s| s.ignore_invalid)
+            {
+                option_was_stripped = false;
+                field_type
+            } else {
+                return Err(Error::new_spanned(
+                    field_type,
+                    "can't `strip_option` - field is not `Option<...>`",
+                ));
+            }
         } else {
+            option_was_stripped = false;
             field_type
         };
         let (arg_type, arg_expr) = if field.builder_attr.setter.auto_into.is_some() {
@@ -283,24 +304,46 @@ impl<'a> StructInfo<'a> {
             .as_ref()
             .and_then(|strip_bool| strip_bool.fallback.as_ref())
             .map(|fallback| (fallback.clone(), quote!(#field_name: #field_type), quote!(#arg_expr)));
-        let strip_option_fallback = field
-            .builder_attr
-            .setter
-            .strip_option
-            .as_ref()
-            .and_then(|strip_option| strip_option.fallback.as_ref())
-            .map(|fallback| (fallback.clone(), quote!(#field_name: #field_type), quote!(#arg_expr)));
 
-        let (param_list, arg_expr) = if field.builder_attr.setter.strip_bool.is_some() {
-            (quote!(), quote!(true))
+        let strip_option_fallback = field.builder_attr.setter.strip_option.as_ref().and_then(|strip_option| {
+            if let Some(ref fallback) = strip_option.fallback {
+                Some((fallback.clone(), quote!(#field_name: #field_type), quote!(#arg_expr)))
+            } else if strip_option.fallback_prefix.is_none() && strip_option.fallback_suffix.is_none() {
+                None
+            } else {
+                let method = strip_raw_ident_prefix(field_name.to_string());
+                let prefix = strip_option.fallback_prefix.as_deref().unwrap_or_default();
+                let suffix = strip_option.fallback_suffix.as_deref().unwrap_or_default();
+                let fallback_name = syn::Ident::new(&format!("{}{}{}", prefix, method, suffix), field_name.span());
+                Some((fallback_name, quote!(#field_name: #field_type), quote!(#arg_expr)))
+            }
+        });
+
+        let (method_generics, param_list, arg_expr, method_where_clause) = if field.builder_attr.setter.strip_bool.is_some() {
+            (quote!(), quote!(), quote!(true), quote!())
         } else if let Some(transform) = &field.builder_attr.setter.transform {
             let params = transform.params.iter().map(|(pat, ty)| quote!(#pat: #ty));
             let body = &transform.body;
-            (quote!(#(#params),*), quote!({ #body }))
-        } else if field.builder_attr.setter.strip_option.is_some() {
-            (quote!(#field_name: #arg_type), quote!(Some(#arg_expr)))
+            let method_generics = transform.generics.as_ref().map_or(quote!(), |g| g.to_token_stream());
+            let method_where_clause = transform
+                .generics
+                .as_ref()
+                .and_then(|g| g.where_clause.as_ref())
+                .map_or(quote!(), |w| w.to_token_stream());
+
+            let body = match &transform.return_type {
+                syn::ReturnType::Default => quote!({ #body }),
+                syn::ReturnType::Type(_, ty) => quote!({
+                    let value: #ty = { #body };
+                    value
+                }),
+            };
+
+            (method_generics, quote!(#(#params),*), body, method_where_clause)
+        } else if option_was_stripped {
+            (quote!(), quote!(#field_name: #arg_type), quote!(Some(#arg_expr)), quote!())
         } else {
-            (quote!(#field_name: #arg_type), arg_expr)
+            (quote!(), quote!(#field_name: #arg_type), arg_expr, quote!())
         };
 
         let repeated_fields_error_type_name = syn::Ident::new(
@@ -320,7 +363,9 @@ impl<'a> StructInfo<'a> {
                 #deprecated
                 #doc
                 #[allow(clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
-                pub fn #method_name (self, #param_list) -> #builder_name <#target_generics> {
+                pub fn #method_name #method_generics (self, #param_list) -> #builder_name <#target_generics>
+                #method_where_clause
+                {
                     let #field_name = (#arg_expr,);
                     let ( #(#destructuring,)* ) = self.fields;
                     #builder_name {
@@ -338,7 +383,9 @@ impl<'a> StructInfo<'a> {
                 #deprecated
                 #doc
                 #[allow(clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
-                pub fn #method_name (self, #param_list) -> #builder_name <#target_generics> {
+                pub fn #method_name #method_generics (self, #param_list) -> #builder_name <#target_generics>
+                #method_where_clause
+                {
                     let #field_name = (#arg_expr,);
                     let ( #(#destructuring,)* ) = self.fields;
                     #builder_name {
@@ -358,7 +405,9 @@ impl<'a> StructInfo<'a> {
                 #deprecated
                 #doc
                 #[allow(clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
-                pub fn #method_name (self, #param_list) -> #builder_name <#target_generics> {
+                pub fn #method_name #method_generics (self, #param_list) -> #builder_name <#target_generics>
+                #method_where_clause
+                {
                     let #field_name = (#arg_expr,);
                     let ( #(#destructuring,)* ) = self.fields;
                     #builder_name {
@@ -381,7 +430,9 @@ impl<'a> StructInfo<'a> {
                     note = #repeated_fields_error_message
                 )]
                 #doc
-                pub fn #method_name (self, _: #repeated_fields_error_type_name) -> #builder_name <#target_generics> {
+                pub fn #method_name #method_generics (self, _: #repeated_fields_error_type_name) -> #builder_name <#target_generics>
+                #method_where_clause
+                {
                     self
                 }
             }
@@ -389,11 +440,9 @@ impl<'a> StructInfo<'a> {
     }
 
     fn required_field_impl(&self, field: &FieldInfo) -> TokenStream {
-        let StructInfo { ref builder_name, .. } = self;
+        let StructInfo { builder_name, .. } = &self;
 
-        let FieldInfo {
-            name: ref field_name, ..
-        } = field;
+        let FieldInfo { name: field_name, .. } = field;
         let mut builder_generics: Vec<syn::GenericArgument> = self
             .generics
             .params
@@ -587,26 +636,7 @@ impl<'a> StructInfo<'a> {
             let mut generics = self.generics.clone();
             for field in self.included_fields() {
                 if field.builder_attr.default.is_some() {
-                    let trait_ref = syn::TraitBound {
-                        paren_token: None,
-                        lifetimes: None,
-                        modifier: syn::TraitBoundModifier::None,
-                        path: {
-                            let mut path = self.builder_attr.crate_module_path.clone();
-                            path.segments.push(syn::PathSegment {
-                                ident: Ident::new("Optional", Span::call_site()),
-                                arguments: syn::PathArguments::AngleBracketed(syn::AngleBracketedGenericArguments {
-                                    colon2_token: None,
-                                    lt_token: Default::default(),
-                                    args: [syn::GenericArgument::Type(field.ty.clone())].into_iter().collect(),
-                                    gt_token: Default::default(),
-                                }),
-                            });
-                            path
-                        },
-                    };
-                    let mut generic_param: syn::TypeParam = field.generic_ident.clone().into();
-                    generic_param.bounds.push(trait_ref.into());
+                    let generic_param: syn::TypeParam = field.generic_ident.clone().into();
                     generics.params.push(generic_param.into());
                 }
             }
@@ -616,6 +646,8 @@ impl<'a> StructInfo<'a> {
 
         let (_, ty_generics, where_clause) = self.generics.split_for_impl();
 
+        let name_with_generics = quote!(#name #ty_generics);
+
         let modified_ty_generics = modify_types_generics_hack(&ty_generics, |args| {
             args.push(syn::GenericArgument::Type(
                 type_tuple(self.included_fields().map(|field| {
@@ -631,32 +663,99 @@ impl<'a> StructInfo<'a> {
 
         let destructuring = self.included_fields().map(|f| f.name);
 
+        let crate_module_path = &self.builder_attr.crate_module_path;
+
+        let where_predicates_for_defaults: Vec<syn::WherePredicate> = self.fields.iter().enumerate()
+            .filter(|(_, field)| field.builder_attr.default.is_some() && field.builder_attr.setter.skip.is_none())
+            .map(|(field_index, field)| {
+                let types = self.fields.iter().take(field_index).map(|dep_field| {
+                    let dep_type = dep_field.ty;
+                    let dep_mut = dep_field.maybe_mut();
+                    quote!(&'__typed_builder_lifetime_for_default #dep_mut #dep_type)
+                }).chain(core::iter::once({
+                    let generic_argument: syn::Type = field.type_ident();
+                    quote!(#generic_argument)
+                }));
+                let field_type = field.ty;
+                parse_quote!{
+                    #name_with_generics: for<'__typed_builder_lifetime_for_default> #crate_module_path::NextFieldDefault<(#(#types,)*), Output = #field_type>
+                }
+
+            }).collect::<Vec<_>>();
+        let where_clause_storage;
+        let where_clause = if where_predicates_for_defaults.is_empty() {
+            where_clause
+        } else {
+            let mut predicates: Punctuated<_, _> = Default::default();
+            if let Some(where_clause) = where_clause {
+                predicates.extend(where_clause.predicates.iter().cloned());
+            }
+            predicates.extend(where_predicates_for_defaults);
+            where_clause_storage = syn::WhereClause {
+                where_token: Default::default(),
+                predicates,
+            };
+            Some(&where_clause_storage)
+        };
+
         // The default of a field can refer to earlier-defined fields, which we handle by
         // writing out a bunch of `let` statements first, which can each refer to earlier ones.
         // This means that field ordering may actually be significant, which isn't ideal. We could
         // relax that restriction by calculating a DAG of field default dependencies and
         // reordering based on that, but for now this much simpler thing is a reasonable approach.
-        let assignments = self.fields.iter().map(|field| {
-            let name = &field.name;
+        let assignments = self
+            .fields
+            .iter()
+            .enumerate()
+            .map(|(field_index, field)| {
+                let name = &field.name;
+                let maybe_mut = field.maybe_mut();
 
-            let maybe_mut = if let Some(span) = field.builder_attr.mutable_during_default_resolution {
-                quote_spanned!(span => mut)
-            } else {
-                quote!()
-            };
+                if let Some(ref default) = field.builder_attr.default {
+                    if field.builder_attr.setter.skip.is_some() {
+                        let make_fields_refs = self.fields.iter().take(field_index).map(|dep_field| {
+                            let dep_name = dep_field.name;
+                            let dep_mut = dep_field.maybe_mut();
+                            quote! {
+                                #[allow(unused_variables)]
+                                let #dep_name = &#dep_mut #dep_name;
+                            }
+                        });
+                        quote! {
+                            let #maybe_mut #name = {
+                                #(#make_fields_refs)*
+                                #default
+                            };
+                        }
+                    } else {
+                        let (types, values): (Vec<_>, Vec<_>) = self
+                            .fields
+                            .iter()
+                            .take(field_index)
+                            .map(|dep_field| {
+                                let dep_type = dep_field.ty;
+                                let dep_name = dep_field.name;
+                                let dep_mut = dep_field.maybe_mut();
+                                (quote!(&#dep_mut #dep_type), quote!(&#dep_mut #dep_name))
+                            })
+                            .chain(core::iter::once({
+                                let generic_argument: syn::Type = field.type_ident();
+                                (quote!(#generic_argument), quote!(#name))
+                            }))
+                            .unzip();
 
-            if let Some(ref default) = field.builder_attr.default {
-                if field.builder_attr.setter.skip.is_some() {
-                    quote!(let #maybe_mut #name = #default;)
+                        quote! {
+                            let #maybe_mut #name = <
+                                #name_with_generics
+                                as
+                                #crate_module_path::NextFieldDefault<(#(#types,)*)>>::resolve((#(#values,)*));
+                        }
+                    }
                 } else {
-                    let crate_module_path = &self.builder_attr.crate_module_path;
-
-                    quote!(let #maybe_mut #name = #crate_module_path::Optional::into_value(#name, || #default);)
+                    quote!(let #maybe_mut #name = #name.0;)
                 }
-            } else {
-                quote!(let #maybe_mut #name = #name.0;)
-            }
-        });
+            })
+            .collect::<Vec<_>>();
         let field_names = self.fields.iter().map(|field| field.name);
 
         let build_method_name = self.build_method_name();
@@ -686,7 +785,7 @@ impl<'a> StructInfo<'a> {
         };
 
         quote!(
-            #[allow(dead_code, non_camel_case_types, missing_docs)]
+            #[allow(dead_code, non_camel_case_types, missing_docs, clippy::ref_option_ref)]
             #[automatically_derived]
             impl #impl_generics #builder_name #modified_ty_generics #where_clause {
                 #build_method_doc
@@ -706,14 +805,23 @@ impl<'a> StructInfo<'a> {
 
     pub fn derive(&self) -> syn::Result<TokenStream> {
         let builder_creation = self.builder_creation_impl()?;
+
         let fields = self
             .setter_fields()
             .map(|f| self.field_impl(f))
             .collect::<Result<TokenStream, _>>()?;
+
+        let next_field_default_impls = self
+            .fields
+            .iter()
+            .filter_map(|field| field.gen_next_field_default_trait_impl(self).transpose())
+            .collect::<Result<TokenStream, _>>()?;
+
         let required_fields = self
             .setter_fields()
             .filter(|f| f.builder_attr.default.is_none())
             .map(|f| self.required_field_impl(f));
+
         let mutators = self
             .fields
             .iter()
@@ -721,11 +829,13 @@ impl<'a> StructInfo<'a> {
             .chain(&self.builder_attr.mutators)
             .map(|m| self.mutator_impl(m))
             .collect::<Result<TokenStream, _>>()?;
+
         let build_method = self.build_method_impl();
 
         Ok(quote! {
             #builder_creation
             #fields
+            #next_field_default_impls
             #(#required_fields)*
             #mutators
             #build_method
